@@ -1,9 +1,5 @@
-#ifndef __OBJBAR_H
-#define __OBJBAR_H
-
-#include "ywindow.h"
-#include "ybutton.h"
-#include "obj.h"
+#ifndef OBJBAR_H
+#define OBJBAR_H
 
 class ObjectButton;
 
