@@ -478,6 +478,14 @@ bool YFrameClient::handleTimer(YTimer* timer) {
             }
         }
     }
+    if (fUrgencyTimer == timer) {
+        fUrgencyTimer = null;
+        xsmart<XWMHints> h(XGetWMHints(xapp->display(), handle()));
+        if (h && hasbit(h->flags, XUrgencyHint)) {
+            h->flags &= ~XUrgencyHint;
+            XSetWMHints(xapp->display(), handle(), h);
+        }
+    }
 
     return false;
 }
@@ -660,19 +668,7 @@ void YFrameClient::handleProperty(const XPropertyEvent &property) {
 
     case XA_WM_HINTS:
         if (new_prop) prop.wm_hints = true;
-        {
-            Drawable oldPix = iconPixmapHint();
-            Drawable oldMask = iconMaskHint();
-            bool oldUrge = urgencyHint();
-            getWMHints();
-            if (oldPix != iconPixmapHint() || oldMask != iconMaskHint()) {
-                refreshIcon();
-            }
-            if (oldUrge != urgencyHint()) {
-                if (getFrame())
-                    getFrame()->setWmUrgency(urgencyHint());
-            }
-        }
+        updateWMHints();
         prop.wm_hints = new_prop;
         break;
 
@@ -959,8 +955,8 @@ void YFrameClient::handleClientMessage(const XClientMessageEvent &message) {
                message.format == 32)
     {
         if (getFrame())
-            getFrame()->startMoveSize(message.data.l[0], message.data.l[1],
-                                      message.data.l[2]);
+            getFrame()->netMoveSize(message.data.l[0], message.data.l[1],
+                                    message.data.l[2]);
     } else if (message.message_type == _XA_NET_MOVERESIZE_WINDOW) {
         if (getFrame()) {
             long flag = message.data.l[0];
@@ -1284,6 +1280,33 @@ Pixmap YFrameClient::iconMaskHint() const {
     return wmHint(IconMaskHint) ? fHints->icon_mask : None;
 }
 
+void YFrameClient::updateWMHints() {
+    Drawable oldPix = iconPixmapHint();
+    Drawable oldMask = iconMaskHint();
+    bool oldUrgency = urgencyHint();
+    getWMHints();
+    if (oldPix != iconPixmapHint() || oldMask != iconMaskHint()) {
+        refreshIcon();
+    }
+    if (fUrgencyTimer) {
+        if (urgencyHint() == false)
+            fUrgencyTimer = null;
+        else if (getFrame() && getFrame()->focused())
+            fHints->flags &= ~XUrgencyHint;
+    }
+    if (oldUrgency != urgencyHint()) {
+        if (getFrame())
+            getFrame()->setWmUrgency(urgencyHint());
+    }
+}
+
+void YFrameClient::clearUrgency() {
+    if (urgencyHint()) {
+        fHints->flags &= ~XUrgencyHint;
+        fUrgencyTimer->setTimer(500L, this, true);
+    }
+}
+
 bool YFrameClient::isDockApp() const {
     return isDockAppIcon() || isDockAppWindow();
 }
@@ -1808,7 +1831,7 @@ bool YFrameClient::getNetWMUserTime(Window window, unsigned long &time) {
     if (!prop.net_wm_user_time && !prop.net_wm_user_time_window)
         return false;
 
-    YProperty prop(this, _XA_NET_WM_USER_TIME, F32, 1, XA_CARDINAL);
+    YProperty prop(window, _XA_NET_WM_USER_TIME, F32, 1, XA_CARDINAL);
     if (prop) {
         MSG(("got user time"));
         time = *prop & 0xffffffff;
