@@ -11,6 +11,7 @@
 #include <X11/Xatom.h>
 
 const char DockApp::propertyName[] = "_ICEWM_DOCKAPPS";
+extern bool limitByDockLayer;
 
 DockApp::DockApp():
     YWindow(nullptr, None,
@@ -86,6 +87,7 @@ bool DockApp::setup() {
     XChangeProperty(xapp->display(), handle(), XA_WM_CLASS, XA_STRING, 8,
                     PropModeReplace, wmClassName, sizeof(wmClassName));
     setNetWindowType(_XA_NET_WM_WINDOW_TYPE_DOCK);
+    setProperty(_XA_WIN_LAYER, XA_CARDINAL, layered);
     if (intern == None) {
         intern = xapp->atom(propertyName);
     }
@@ -203,6 +205,8 @@ bool DockApp::dock(YFrameClient* client) {
             client->setDocked(true);
             direction = +1;
             retime();
+            if (layered == WinLayerInvalid)
+                setup();
         }
         else {
             XRemoveFromSaveSet(xapp->display(), icon);
@@ -263,13 +267,17 @@ bool DockApp::handleTimer(YTimer* t) {
 }
 
 void DockApp::adapt() {
+    const unsigned wold = int(visible()) * width();
     if (docks.nonempty()) {
+        int sx, sy;
+        unsigned sw, sh;
+        desktop->getScreenGeometry(&sx, &sy, &sw, &sh);
         int mx, my, Mx, My;
         manager->getWorkArea(&mx, &my, &Mx, &My);
         int rows = min(docks.getCount(), (My - my) / 64);
         int cols = (docks.getCount() + (rows - 1)) / rows;
         rows = (docks.getCount() + (cols - 1)) / cols;
-        int xpos = isRight ? Mx - cols * 64 : 0;
+        int xpos = isRight ? sx + int(sw) - cols * 64 : sx;
         int ypos = (center == -1) ? 0
                  : (center == +1) ? (My - rows * 64)
                  : my + (My - my - rows * 64) / 2;
@@ -335,6 +343,11 @@ void DockApp::adapt() {
     }
     if (timer)
         timer = null;
+
+    if (wold != int(visible()) * width() && layered == WinLayerDock) {
+        if (limitByDockLayer)
+            manager->requestWorkAreaUpdate();
+    }
 }
 
 void DockApp::proper() {
@@ -556,9 +569,7 @@ void DockApp::handleEndDrag(const XButtonEvent& down, const XButtonEvent& up) {
             XMoveWindow(xapp->display(), docks[d].window,
                         (attr.x / 64) * 64 + dragxpos % 64,
                         (attr.y / 64) * 64 + dragypos % 64);
-            docking copy(docks[d]);
-            docks.remove(d);
-            docks.insert(a, copy);
+            docks.moveto(d, a);
             proper();
             retime();
         }
@@ -570,3 +581,19 @@ void DockApp::handleEndDrag(const XButtonEvent& down, const XButtonEvent& up) {
     }
 }
 
+void DockApp::handleClientMessage(const XClientMessageEvent& message) {
+    if (message.message_type == _XA_WIN_LAYER) {
+        long layer = message.data.l[0];
+        if (validLayer(layer) && layer != layered) {
+            bool work = (layer == WinLayerDock || layered == WinLayerDock);
+            layered = int(layer);
+            setProperty(_XA_WIN_LAYER, XA_CARDINAL, layer);
+            if (visible() && docks.nonempty()) {
+                manager->restackWindows();
+            }
+            if (work && limitByDockLayer)
+                manager->requestWorkAreaUpdate();
+        }
+    }
+}
+
