@@ -261,6 +261,7 @@ public:
     void actionWindows(YAction action);
     void toggleDesktop();
     void cascadeWindows();
+    void focusOverlap();
 
     bool haveClients();
     void setupRootProxy();
@@ -318,7 +319,8 @@ public:
     };
 
     const DesktopLayout& layout() const { return fLayout; }
-    bool handleSwitchWorkspaceKey(const XKeyEvent& key, KeySym k, unsigned vm);
+    bool handleSwitchWorkspaceKey(const XKeyEvent& key);
+    bool handleWMKey(const XKeyEvent &key);
 
     int getSwitchScreen();
     bool switchWindowVisible() const;
@@ -339,10 +341,9 @@ private:
     YFrameClient* allocateClient(Window win, bool mapClient);
     YFrameWindow* allocateFrame(YFrameClient* client);
     void updateArea(int workspace, int screen_number, int l, int t, int r, int b);
-    bool handleWMKey(const XKeyEvent &key, KeySym k, unsigned vm);
     void setWmState(WMState newWmState);
+    void updateServerTime(YFrameWindow* frame);
     void refresh();
-    void focusOverlap();
 
     IApp *app;
     YActionListener *wmActionListener;
