@@ -93,34 +93,34 @@ DProgram *DProgram::newProgram(
 KProgramArrayType keyProgs;
 
 KProgram::KProgram(const char *key, DProgram *prog, bool bIsDynSwitchMenuProg) :
-    fDef(newstr(key)),
-    fKey(NoSymbol),
-    fMod(0),
+    wm(newstr(key)),
     bIsDynSwitchMenu(bIsDynSwitchMenuProg),
     fProg(prog),
     pSwitchWindow(nullptr)
 {
-    parse();
-    keyProgs.append(this);
 }
 
-void KProgram::parse() {
-    xapp->parseKey(fDef, &fKey, &fMod);
+KProgram::~KProgram() {
+    delete fProg;
+    if (wm.initial == false)
+        delete[] const_cast<char *>(wm.name);
+    delete pSwitchWindow;
 }
 
 class MenuProgSwitchItems: public ISwitchItems {
     MenuProgMenu *menu;
     int zTarget;
-
-    KeySym key;
-    unsigned int mod;
+    const WMKey* wmkey;
 
 public:
-    MenuProgSwitchItems(DProgram* prog, KeySym key, unsigned keymod) :
-        ISwitchItems(), zTarget(0), key(key), mod(keymod) {
+    MenuProgSwitchItems(DProgram* prog, const WMKey* wmkey) :
+        ISwitchItems(), zTarget(0), wmkey(wmkey) {
         menu = new MenuProgMenu(wmapp, wmapp, nullptr /* no wmaction handling*/,
                 "switch popup internal menu", prog->cmd(), prog->args());
     }
+    ~MenuProgSwitchItems() {
+        delete menu;
+    }
     virtual void updateList() override {
         menu->refresh();
         zTarget = 0;
@@ -131,11 +131,11 @@ public:
     virtual bool isEmpty() override {
         return menu->itemCount() == 0;
     }
-    virtual bool isKey(KeySym k, unsigned int mod) override {
-        return k == this->key && mod == this->mod;
+    virtual bool isKey(const XKeyEvent& key) override {
+        return *wmkey == key;
     }
     unsigned modifiers() override {
-        return mod;
+        return wmkey->mod;
     }
     virtual bool setWMClass(char* wmclass) override {
         if (wmclass) free(wmclass); // unimplemented
@@ -194,7 +194,7 @@ void KProgram::open(unsigned mods) {
 
     if (bIsDynSwitchMenu) {
         if (!pSwitchWindow) {
-            ISwitchItems* items = new MenuProgSwitchItems(fProg, fKey, fMod);
+            ISwitchItems* items = new MenuProgSwitchItems(fProg, &wm);
             pSwitchWindow = new SwitchWindow(desktop, items, quickSwitchVertical);
         }
         pSwitchWindow->begin(true, mods);
