@@ -240,20 +240,23 @@ int YApplication::mainLoop() {
 
     for (fExitLoop = fExitApp; (fExitApp | fExitLoop) == false; ) {
         bool didIdle = handleIdle();
-
+        int nfds = 0;
         fd_set read_fds;
         FD_ZERO(&read_fds);
         fd_set write_fds;
         FD_ZERO(&write_fds);
 
         for (YPollIterType iPoll = polls.iterator(); ++iPoll; ) {
-            PRECONDITION(iPoll->fd() >= 0);
+            const int fd = iPoll->fd();
+            PRECONDITION(fd >= 0);
             if (iPoll->forRead()) {
-                FD_SET(iPoll->fd(), &read_fds);
+                FD_SET(fd, &read_fds);
             }
             if (iPoll->forWrite()) {
-                FD_SET(iPoll->fd(), &write_fds);
+                FD_SET(fd, &write_fds);
             }
+            if (nfds <= fd)
+                nfds = fd + 1;
         }
 
         timeval timeout = {0, 0L};
@@ -266,7 +269,7 @@ int YApplication::mainLoop() {
 #endif
 
         int rc;
-        rc = select(sizeof(fd_set) * 8,
+        rc = select(nfds,
                     SELECT_TYPE_ARG234 &read_fds,
                     SELECT_TYPE_ARG234 &write_fds,
                     nullptr,
@@ -323,10 +326,21 @@ void YApplication::exit(int exitCode) {
 
 void YApplication::handleSignal(int sig) {
     if (sig == SIGCHLD) {
-        int s, rc;
-        do {
-            rc = waitpid(-1, &s, WNOHANG);
-        } while (rc > 0);
+        for (int pid = 1; pid > 0 || (pid == -1 && errno == EINTR); ) {
+            int status = errno = 0;
+            pid = waitpid(-1, &status, WNOHANG);
+            if (pid > 0) {
+                for (int i = waits.getCount(); 0 <= --i; ) {
+                    if (i < waits.getCount()) {
+                        if (waits[i].pid == pid) {
+                            YPidWaiter* waiter = waits[i].waiter;
+                            waits.remove(i);
+                            waiter->waitCompleted(pid, status);
+                        }
+                    }
+                }
+            }
+        }
     }
 }
 
@@ -389,7 +403,7 @@ void YApplication::closeFiles() {
 #endif
 }
 
-int YApplication::runProgram(const char *path, const char *const *args) {
+int YApplication::runProgram(const char *path, const char *const *args, int fd) {
     flushXEvents();
 
     int cpid = -1;
@@ -408,6 +422,10 @@ int YApplication::runProgram(const char *path, const char *const *args) {
         if (open("/dev/null", O_RDONLY) != 0)
             _exit(1);
 #endif
+        if (2 < fd) {
+            dup2(fd, 1);
+            close(fd);
+        }
         closeFiles();
 
         if (args)
@@ -421,13 +439,11 @@ int YApplication::runProgram(const char *path, const char *const *args) {
     return cpid;
 }
 
-int YApplication::waitProgram(int p) {
-    int rc = -1, status = 0;
-    if (p > 0) {
-        do {
-            errno = status = 0;
-            rc = waitpid(p, &status, 0);
-        } while (rc == -1 && errno == EINTR);
+int YApplication::waitProgram(int pid) {
+    int rc = -1, status = errno = 0;
+    if (pid > 0) {
+        while ((rc = waitpid(pid, &status, 0)) == -1 && errno == EINTR)
+            status = errno = 0;
     }
     return rc == -1 ? -1 : status;
 }
@@ -469,6 +485,26 @@ void YSignalPoll::notifyRead()
 }
 #endif
 
+void YApplication::registerWait(int pid, YPidWaiter* waiter) {
+    waits += WaitHandler(pid, waiter);
+}
+
+void YApplication::unregisterWait(YPidWaiter* waiter) {
+    for (int i = waits.getCount(); 0 <= --i; ) {
+        if (waiter == waits[i].waiter) {
+            waits.remove(i);
+        }
+    }
+}
+
+YPidWaiter::~YPidWaiter() {
+    mainLoop->unregisterWait(this);
+}
+
+void YPidWaiter::waitForPid(int pid) {
+    mainLoop->registerWait(pid, this);
+}
+
 const upath& YApplication::getLibDir() {
     static upath dir( LIBDIR );
     return dir;
@@ -479,34 +515,34 @@ const upath& YApplication::getConfigDir() {
     return dir;
 }
 
-const upath& YApplication::getPrivConfDir() {
+const upath& YApplication::getPrivConfDir(bool create) {
     static upath dir;
     if (dir.isEmpty()) {
         const char *env = getenv("ICEWM_PRIVCFG");
-        if (env) {
+        if (nonempty(env))
             dir = env;
-            if ( ! dir.dirExists())
-                dir.mkdir();
-        }
         else {
+            const upath home = getHomeDir();
             env = getenv("XDG_CONFIG_HOME");
-            if (env)
-                dir = env;
-            else {
-                dir = getHomeDir() + "/.config";
-            }
-            dir += "/icewm";
-            if (!dir.dirExists()) {
-                dir = getHomeDir() + "/.icewm";
-                if ( ! dir.dirExists())
-                    dir.mkdir();
-            }
+            if (nonempty(env))
+                dir = upath(env) + "/icewm";
+            else if (home != null)
+                dir = home + "/.config/icewm";
+            if ( !dir.dirExists())
+                dir = home + "/.icewm";
         }
         MSG(("using %s for private configuration files", dir.string()));
     }
+    if (create && dir.nonempty()) {
+        dir.ensureDirectory();
+    }
     return dir;
 }
 
+upath YApplication::getPrivConfFile(mstring basename) {
+    return getPrivConfDir() + basename;
+}
+
 upath YApplication::getHomeDir() {
     char *env = getenv("HOME");
     if (nonempty(env)) {
