@@ -90,7 +90,8 @@ void YMsgBox::autoSize() {
     unsigned ph = (fPixmap != null) ? fPixmap->height() : 0;
     unsigned lw = fLabel ? fLabel->width() : 0;
     unsigned lh = fLabel ? fLabel->height() : 0;
-    unsigned w = clamp(lw + pw + 2*margin, 240U, desktop->width());
+    unsigned mw = fInput ? 300U : 240U;
+    unsigned w = clamp(lw + pw + 2*margin, mw, desktop->width());
     unsigned h = margin;
 
     if (fLabel) {
@@ -165,7 +166,7 @@ void YMsgBox::handleClose() {
     if (fListener)
         fListener->handleMsgBox(this, mbClose);
     else
-        unmanage();
+        delete this;
 }
 
 void YMsgBox::handleFocus(const XFocusChangeEvent& focus) {
@@ -189,11 +190,7 @@ void YMsgBox::showFocused() {
     become();
 }
 
-void YMsgBox::unmanage() {
-    manager->unmanageClient(this);
-}
-
-void YMsgBox::inputReturn(YInputLine* input) {
+void YMsgBox::inputReturn(YInputLine* input, bool control) {
     if (fListener) {
         fListener->handleMsgBox(this, mbOK);
     }
